#!/usr/bin/perl -w

# Copyright 2014, 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Usage: perl centroid-in-join.pl
#
# Create some files
#
#     /tmp/centroid-001.png
#
# etc which are images of where the join area centroid is relative to the R5
# dragon curve.  View them with for example
#
#     xzgv /tmp/centroid-*.png
#
# The square containing the centroid is in the middle of each image.
#
# $want_rotate option below does a rotation by atan(2,1)*$k so that the
# curve start to end is horizontal.  The way the square develops is less
# clear that way.

use 5.010;
use strict;
use Geometry::AffineTransform;
use Image::Base::GD;
use Math::BaseCnv 'cnv';
use Math::PlanePath::R5DragonCurve 118; # v.118 for xyxy_to_n_either()
use Math::BigRat;
use Math::BigFloat;
use POSIX 'ceil';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyR5Dragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $want_rotate = 0;

my $square_scale = 56;  # squares across picture
my $pixel_scale = 6;   # pixels per square
my $circle_size = .15;  # fraction of a square
my $dot_size = .1;      # fraction of a square

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

sub numify {
  my ($x) = @_;
  return (ref $x ? $x->numify : $x);
}
sub numify_small {
  my ($x) = @_;
  return (abs($x) < 0xFF_FFFF ? numify($x) : $x);
}

my $path = Math::PlanePath::R5DragonCurve->new;
my @values;
foreach my $k (1 .. 100) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);

  my ($x,$y) = GJ_xy($k);
  my $x1 = int($x);                          # x1,y1,x2,y2 square
  my $x2 = ($x < $x1 ? $x1 - 1 : $x1 + 1);   # containing centroid
  my $y1 = int($y);
  my $y2 = ($y < $y1 ? $y1 - 1 : $y1 + 1);

  if ($x1 > $x2) { ($x1,$x2) = ($x2,$x1) }
  if ($y1 > $y2) { ($y1,$y2) = ($y2,$y1) }

  $x1 = numify_small($x1);
  $y1 = numify_small($y1);
  $x2 = numify_small($x2);
  $y2 = numify_small($y2);

  my @n_list = ($path->xyxy_to_n_either($x1,$y1, $x2,$y1),
                $path->xyxy_to_n_either($x1,$y2, $x2,$y2),
                $path->xyxy_to_n_either($x1,$y1, $x1,$y2),
                $path->xyxy_to_n_either($x2,$y1, $x2,$y2));
  @n_list = grep {defined} @n_list;
  @n_list = grep {$_ < $n_hi} @n_list;
  my $num = scalar(@n_list);

  print "k=$k  num=$num  ";
  foreach my $n (@n_list) {
    my $n5 = cnv($n,10,5);
    print "[$n5],";
  }
  my $xfrac = numify($x - $x1);
  my $yfrac = numify($y - $y1);
  print "  $x1 $y1 frac $xfrac $yfrac";
  # if ($num) {
  #   print "  $x $y  $xs $ys    $x1,$y1  $x2,$y2";
  # }
  print "\n";

  if ($k > 0 && $k < 50) {
    my $affine = Geometry::AffineTransform->new;
    my $xm = ( ($x1 + $x2)/2 );  # middle of centroid square
    my $ym = ( ($y1 + $y2)/2 );

    $affine->translate(numify_small(-$xm), numify_small(-$ym));
    if ($want_rotate) {
      $affine->rotate(-atan2(2,1)*$k);
    }
    $affine->translate($square_scale/2, - $square_scale/2);

    $affine->scale($pixel_scale,-$pixel_scale);

    my $width = $square_scale*$pixel_scale;
    my $height = $square_scale*$pixel_scale;
    my $image = Image::Base::GD->new(-width => $width,
                                     -height => $height);
    $image->rectangle (0,0, $width-1,$height-1, 'black', 1);
    {
      my ($wx,$wy) = $affine->transform($x,$y);
      $wx = numify($wx);
      $wy = numify($wy);
      $image->ellipse ($wx - $pixel_scale*$circle_size,
                       $wy - $pixel_scale*$circle_size,
                       $wx + $pixel_scale*$circle_size,
                       $wy + $pixel_scale*$circle_size,
                       'red');
    }
    my $x_limit = numify_small($x1 + ceil($square_scale/2));
    my $y_start = numify_small($y1 - ceil($square_scale/2));
    my $y_limit = numify_small($y1 + ceil($square_scale/2));
    for (my $x = numify_small($x1 - ceil($square_scale/2)); $x <= $x_limit; $x++) {
      for (my $y = $y_start; $y <= $y_limit + 6; $y++) {
        # print "$x,$y\n";
        foreach my $vert (0,1) {
          my $tx = $x;
          my $ty = $y;
          if ($vert) { $ty += 1; } else { $tx += 1; }
          my $n = $path->xyxy_to_n_either($x,$y, $tx,$ty) // next;
          $n < $n_hi or next;

          my $colour = ($n % 2 ? 'blue' : 'white');
          $colour = 'white';
          $colour = ($n < $n_hi/2 ? 'white' : 'blue');
          $image->line($affine->transform($x,$y),
                       $affine->transform($tx,$ty),
                       $colour);

          if (($x+$y) % 2) {
            my ($wx,$wy) = $affine->transform($x,$y);
            $wx = numify($wx);
            $wy = numify($wy);
            $image->ellipse ($wx - $pixel_scale*$dot_size,
                             $wy - $pixel_scale*$dot_size,
                             $wx + $pixel_scale*$dot_size,
                             $wy + $pixel_scale*$dot_size,
                             'green', 1);
          }
        }
      }
    }
    $image->rectangle (0,0, 10,10, $num==4 ? 'green' : 'red', 1);
    $image->save(sprintf "/tmp/centroid-%02d.png", $k);
  }
}

require Math::OEIS::Grep;
Math::OEIS::Grep->search (array => \@values,verbose=>1);
exit 0;
