#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019, 2020, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Usage: make-oeis-samples.pl filename filename... >outfile.gp
#
# Generate Pari/GP code making definitions of OEIS sequence sample values,
# such as
#
#     A123456_samples = [10,20,30,40];
#
# The given files are read and all A-numbers found appearing as symbols
# "A123456_samples" are printed to stdout as such definitions.
#
# Values are taken from the OEIS "stripped" file.  That file should
# be ~/OEIS/stripped as described in Math::OEIS::Stripped.  The OEIS samples
# for each A-number are usually about 220 characters, but might be smaller
# when hard to compute, not known, a finite sequence, or next term too big
# to show.
#
# If the stripped file doesn't contain a sequence, then individual
# downloaded sequence files are tried using Math::NumSeq::OEIS.  This might
# be useful for a draft or recently added sequence not in the stripped yet,
# or if you're only interested in a few sequences and so prefer individual
# data files over the full big stripped file.  Currently, individual
# sequence files actually use the Math::NumSeq::OEIS::File module which is
# an undocumented internal.  Not sure if want to document it, or instead
# make a parameter to Math::NumSeq::OEIS.
#
# The Pari/GP code output is printed to standard output and usually should
# be directed to a file where test code can then read it.  The output is
# sorted by A-number.
#
# The input files can be .gp files, or can be .tex or similar containing GP
# code in comments for gp-inline or similar.  The form "A123456_samples" is
# sought anywhere in the input files.  It should be unlikely to occur in
# other text or comments but a few extras in the output won't hurt.
#
# Other Ways:
#
# Another way would be some GP code so OEIS_samples("A123456") would get the
# relevant samples by reading the stripped file or other files when needed.
# (Experiment for that in "my-oeis.gp" with a Perl helper and not yet
# published.)  It could system() to spawn a helper program, or by an
# install() running C code reading.  Though both wouldn't run in GP "secure"
# mode.  This would re-read each time samples are used.  C code re-reading
# could be fast enough for most uses.  A generated file has the attraction
# of users not needing anything extra to run tests, and of being safe
# against minor or major changes to sequences (until regenerated).
#
# One thing not included in the samples is the sequence "OFFSET" (the index
# n of the first term).  Often this is not particularly important, just the
# values, especially when a "list" of some values rather than a function.
#


use 5.006; # for 3-arg open
use strict;
use warnings;
use FindBin;
use Math::OEIS::Stripped;

# uncomment this to run the ### lines
# use Smart::Comments;


print "\\\\ Generated by $FindBin::Script -- DO NOT EDIT\n";

my %want_anums;
foreach my $filename (@ARGV) {
  open my $fh, '<', $filename or die "Cannot open $filename: $!\n";
  while (my $line = readline $fh) {
    while ($line =~ /\b(A\d{6,7})_samples\b/g) {
      $want_anums{$1} ||= "$filename:$.:";
    }
  }
  close $fh or die $!;
}

my $num_anums = scalar(keys %want_anums);
print "\\\\ Total $num_anums A-numbers\n";
print "\\\\\n";
print "\\\\ Extracted from the OEIS \"stripped\" file, http://oeis.org/allfiles.html\n";
print "\\\\ See http://oeis.org/wiki/The_OEIS_End-User_License_Agreement\n";
print "\\\\ for terms of use (at the time of writing Creative Commons\n";
print "\\\\ Attribution Non-Commercial 3.0)\n";
print "\n";

# sorted numerically
my @want_anums = keys %want_anums;
@want_anums = sort {substr($a,1) <=> substr($b,1)} @want_anums;

foreach my $anum (sort keys %want_anums) {
  my $str = Math::OEIS::Stripped->anum_to_values_str($anum);

  # or read from individual A-number files
  if (! defined $str) {
    require Math::NumSeq::OEIS::File;
    my $seq;
    if (eval { $seq = Math::NumSeq::OEIS::File->new (anum => $anum,
                                                     _dont_use_bfile => 1) }) {
      ### using Math-NumSeq-OEIS-File ...
      $str = '';
      my $join = '';
      while (my ($i,$value) = $seq->next) {
        last if length($str) > 1000;
        $str .= $join . $value;
        $join = ',';
      }
    }
  }

  if (! defined $str) {
    my $location = $want_anums{$anum};
    print STDERR "$location values not found for $anum\n";
    exit 1;
  }

  $str =~ s/,+$//;
  print "${anum}_samples = [$str];\n";
}
exit 0;
